
//
// UniqueURL creates a unique URL for get requests that only care about the
// first few characters, the remaining part of the URL is the time in seconds
// since 1970 and a "fetchCounter", so that we never request the same URL twice.
//

function UniqueURL( baseUrl ) {
  this.fetchCounter = 0;
  this.baseUrl = baseUrl;
}

UniqueURL.prototype.get = function() {
  var d = new Date();
  this.fetchCounter = this.fetchCounter + 1;
  return( this.baseUrl + d.getTime() + "" + this.fetchCounter );
}


//
// NeatTools an object that interfaces to neattools XServer external module
//

function NeatTools_XServer( urlGenerator ) {
  this.urlGenerator = urlGenerator;
  try {
    this.xmlhttp = new XMLHttpRequest();
  } catch (e) {
    this.xmlhttp=false
  }
  if (!this.xmlhttp) {
    try {
      this.xmlhttp=new ActiveXObject("Msxml2.XMLHTTP")
    } catch (e) {
      try {
        this.xmlhttp=new ActiveXObject("Microsoft.XMLHTTP")
      } catch (E) {
        this.xmlhttp=false
      }
    }
  }
}

NeatTools_XServer.prototype.query = function() {
  try {
    this.xmlhttp.open("GET", this.urlGenerator.get(),true);
    this.xmlhttp.onreadystatechange=function() {
      if (neattools.xmlhttp.readyState==4) {
        this.loadValues(neattools.xmlhttp.responseText);
     }
    }
    this.xmlhttp.send(null)

    // don't repeat if we ever get an exception
    setTimeout("queryNeattools();",1000);
  } catch (e ) {
    alert( "GET exception" );
    for ( x in e ) {
      alert( "exception has: " + x + ", value: " + e[x]);
    }
  }
}

NeatTools_XServer.prototype.loadValues = function loadValues( s ) {
  //cell = document.getElementById( "n1x" );
  //cell = document.getElementById( "n1y" );
  //additional = "";
  while ( s.indexOf( '>' ) > 0 ) {
    //additional = additional + " skipped one, ";
    s = s.substring( s.indexOf( '>' ) + 1 );
  }
  //cell.innerText = s;
  outputval = "";
  slen = s.length;
  counter = 0;
  maplist[0] = 0;
  maplist[1] = 0; 
  maplist[2] = 0; 
  maplist[3] = 0; 
  maplist[4] = 0; 
  maplist[5] = 0; 
  maplist[6] = 0; 
  maplist[7] = 0; 
  maplist[8] = 0; 
  i = 0;
  for ( ; i < slen; i++ ) {
    cval = s.charAt( i );
    if ( cval != ',' ) {
      outputval += cval;
    } else {
      maplist[ counter ] = parseInt( outputval );
      counter = counter + 1;
      outputval = "";
      if ( counter == 9 ) {
        i++;
        break;
      }
    }
  }
  if ( outputval.length > 0 ) {
    maplist[ counter ] = parseInt( outputval );
  }
  cell = document.getElementById( "n1lat" );
  outputval = "";
  for ( ; i < slen; i++ ) {
    cval = s.charAt( i );
    if ( cval != ',' ) {
      outputval += cval;
    } else {
      cell.innerText = outputval;
      i++;
      break;
    }
  }
  cell = document.getElementById( "n1long" );
  outputval = "";
  for ( ; i < slen; i++ ) {
    cval = s.charAt( i );
    if ( cval != ',' ) {
      outputval += cval;
    } else {
      cell.innerText = outputval;
    }
  }
  if ( maplist[0] != 0 ) {
    changeMap( mapManager.getView( 3 ));
    n1v.setX( maplist[1] - 16);
    n1v.setY( maplist[2] - 16);
  } else if ( maplist[3] != 0 ) {
    changeMap( mapManager.getView( 2 ));
    n1v.setX( maplist[4] - 16);
    n1v.setY( maplist[5] - 16);
  } else if ( maplist[6] != 0 ) {
    changeMap( mapManager.getView( 1 ));
    n1v.setX( maplist[7] - 16);
    n1v.setY( maplist[8] - 16);
  }
//  if (( maplist[0] == 0 ) && ( maplist[3] == 0 ) && ( maplist[6] == 0 )) {
//    cell.innerText = "Not in map";
//  } else {
//    cell.innerText = "In map: " + maplist[0] + "," + maplist[3] + "," + maplist[6];
//  }
}
//
// The Map is a wrapper around a dynapi object
//

function Map( mapImage, viewPriority ) {
  this.mapImage = mapImage;
  this.viewPriority = viewPriority;
}

Map.prototype.setView = function( mapView ) {
  this.mapView = mapView;
  mapView.setZIndex( this.viewPriority );
  mapView.setVisible( false );
  mapView.fadeState = 1;
}


//
// GPS_rovers handles the set of images that are associated with moving GPS locations
//
function GPS_rovers( baseUrl, dynapi ) {
  this.baseUrl = baseUrl;
  this.dynapiFunctions = dynapi.functions;
  this.dynapiDocument = dynapi.document;
  this.vehicleList = new Array();
}

GPS_rovers.prototype.addVehicle = function( vehicleFileName, viewPriority ) {
  var dynImage = this.dynapiFunctions.getImage( this.baseUrl + vehicleFileName );

// ..might become an object later
// original line from MapManager, object was:  blah blah ... = new Map( dynImage, viewPriority );
  var vehicle = this.vehicleList[ this.vehicleList.length ] = dynImage; 
  return vehicle;
}


//
// MapManager handles the set of images that are span a fixed range of GPS locations
//

function MapManager( baseUrl, dynapi ) {
  this.baseUrl = baseUrl;
  this.dynapiFunctions = dynapi.functions;
  this.dynapiDocument = dynapi.document;
  this.mapList = new Array();
}

MapManager.prototype.numberMaps = function() {
  return( this.mapList.length );
}

MapManager.prototype.addMap = function( mapFileName, viewPriority ) {
  var dynImage = this.dynapiFunctions.getImage( this.baseUrl + mapFileName );
  var map = this.mapList[ this.mapList.length ] = new Map( dynImage, viewPriority );
  return map;
}


MapManager.prototype.getView = function( mapViewPriority ) {
  for ( var i = 0; i < this.mapList.length; i++ ) {
    m = this.mapList[i];
    if ( m.viewPriority == mapViewPriority ) {
      return( m.mapView );
    }
  }
  return 0;
}
