////////////////// UniqueURL //////////////////////////////////////////////////
//
// UniqueURL creates a unique URL for get requests that only care about the
// first few characters, the remaining part of the URL is the time in seconds
// since 1970 and a "fetchCounter", so that we never request the same URL twice.
//

function UniqueURL( baseUrl ) {
  this.fetchCounter = 0;
  this.baseUrl = baseUrl;
}

UniqueURL.prototype.get = function() {
  var d = new Date();
  this.fetchCounter = this.fetchCounter + 1;
  return( this.baseUrl + d.getTime() + "" + this.fetchCounter );
}


////////////////// NeatTools_XServer ///////////////////////////////////////////
//
// NeatTools_XServer: interfaces to neattools XServer external module
//

function NeatTools_XServer( urlGenerator ) {
  this.urlGenerator = urlGenerator;
  this.numberMaps = 0;
  try {
    this.xmlhttp = new XMLHttpRequest();
  } catch (e) {
    this.xmlhttp=false
  }
  if (!this.xmlhttp) {
    try {
      this.xmlhttp=new ActiveXObject("Msxml2.XMLHTTP")
    } catch (e) {
      try {
        this.xmlhttp=new ActiveXObject("Microsoft.XMLHTTP")
      } catch (E) {
        this.xmlhttp=false
      }
    }
  }
}

var numberMapsExpectedInResponse = 0;
NeatTools_XServer.prototype.query = function( numberMaps ) {
  numberMapsExpectedInResponse = numberMaps;
  try {
    this.xmlhttp.open("GET", this.urlGenerator.get(),true);
    this.xmlhttp.onreadystatechange=function() {
      if (neattools.xmlhttp.readyState==4) {
        this.loadValues(neattools.xmlhttp.responseText);
     }
    }
    this.xmlhttp.send(null)

    // don't repeat if we ever get an exception
    setTimeout("queryNeattools();",1000);
  } catch (e ) {
    //alert( "GET exception" );
    //for ( x in e ) {
     // alert( "exception has: " + x + ", value: " + e[x]);
   // }
  }
}

NeatTools_XServer.prototype.loadValues = function loadValues( s ) {
  while ( s.indexOf( '>' ) > 0 ) {
    s = s.substring( s.indexOf( '>' ) + 1 );
  }
  outputval = "";
  slen = s.length;
  counter = 0;
  llcounter = 0;
  i = 0;
  mapValueCount = numberMapsExpectedInResponse * 3;
  while ( i < slen ) {
    // grab 3 values for each map expected back in query
    outputval = "";
    for ( ; i < slen; i++ ) {
      cval = s.charAt( i );
      if ( cval != ',' ) {
        outputval += cval;
      } else {
        maplist[ counter ] = parseInt( outputval );
        counter = counter + 1;
        outputval = "";
        if (( counter % mapValueCount ) == 0 ) {
          i++;
          break;
        }
      }
    }
    if ( outputval.length > 0 ) {
      maplist[ counter ] = parseInt( outputval );
      counter = counter + 1;
    }

    // grab next two values in latitude and longitude lsits
    outputval = "";
    for ( ; i < slen; i++ ) {
      cval = s.charAt( i );
      if ( cval != ',' ) {
        outputval += cval;
      } else {
        lllist[ llcounter ] = outputval;
        llcounter++;
        i++;
        break;
      }
    }
    outputval = "";
    for ( ; i < slen; i++ ) {
      cval = s.charAt( i );
      if ( cval != ',' ) {
        outputval += cval;
      } else {
        lllist[ llcounter ] = outputval;
        llcounter++;
        i++;
        break;
      }
    }
  }

  // must exist in HTML side...
  setVehicleLocations();
}


////////////////// Map ////////////////////////////////////////////////////////
//
// The Map is a wrapper around a dynapi object
//

function Map( mapImage, viewPriority, neattoolsIndicator ) {
  this.mapImage = mapImage;
  this.viewPriority = viewPriority;
  this.neattoolsIndicator = neattoolsIndicator; // neattools uses this as identifier (implicitly by offset)
}

Map.prototype.setView = function( mapView ) {
  this.mapView = mapView;
  mapView.setZIndex( this.viewPriority );
  mapView.setVisible( false );
  mapView.fadeState = 1;
}


////////////////// Vehicle /////////////////////////////////////////////////////
//
// Vehicle is a moving image on top of a map image
//
function Vehicle( dynImage ) {
  this.dynImage = dynImage;
  this.imageUrl = "";
  this.tracking = false;
  this.view = null;
}

Vehicle.prototype.setTracking = function( val ) {
  this.tracking = val;
}

Vehicle.prototype.setImageUrl = function( imageUrl ) {
  this.imageUrl = imageUrl;
}

Vehicle.prototype.setView = function( v ) {
  this.view = v;
}

Vehicle.prototype.getImage = function() {
  return( this.dynImage );
}

Vehicle.prototype.getView = function() {
  return( this.view );
}


////////////////// GPS_rovers ////////////////////////////////////////////////
//
// GPS_rovers:  manages the set of Vehicles (moving images) associated with 
//  changing GPS locations
//

function GPS_rovers( baseUrl, dynapi ) {
  this.baseUrl = baseUrl;
  this.dynapiFunctions = dynapi.functions;
  this.dynapiDocument = dynapi.document;
  this.vehicleList = new Array();
}

GPS_rovers.prototype.addVehicle = function( vehicleFileName, viewPriority ) {
  var dynImage = this.dynapiFunctions.getImage( this.baseUrl + vehicleFileName );
  var vehicle = this.vehicleList[ this.vehicleList.length ] = new Vehicle( dynImage ); 
  vehicle.setImageUrl( this.baseUrl + vehicleFileName );
  return vehicle;
}

GPS_rovers.prototype.getNumberVehicles = function() {
  return( this.vehicleList.length );
}

GPS_rovers.prototype.getVehicle = function( offset ) {
  return( this.vehicleList[ offset] );
}


//
// load table displaying a vehicle, assumes columns are:
//
//   icon, latitude, longitude
//
GPS_rovers.prototype.loadDisplay = function( table ) {
  numberVehicles = this.vehicleList.length;
  for ( var i = 0; i < numberVehicles; i++ ) {
    row = table.insertRow( table.rows.length );
    cell = row.insertCell( 0 );
    cell.innerHTML = "<input type=\"CHECKBOX\" onclick=\"trackVehicle(this, '" + i + "');\">";
    cell = row.insertCell( 1 );
    vehicle = this.vehicleList[ i ];
    cell.innerHTML = "<img src=\"" + vehicle.imageUrl + "\">";
    cell = row.insertCell( 2 );
    cell.id = "n" + (i+1) + "lat";
    cell.innerHTML = "&nbsp;";
    cell = row.insertCell( 3 );
    cell.id = "n" + (i+1) + "long";
    cell.innerHTML = "&nbsp;";
  }
}

////////////////// MapManager /////////////////////////////////////////////////
//
// MapManager handles the set of images that are span a fixed range of 
// GPS locations
//

function MapManager( baseUrl, dynapi ) {
  this.baseUrl = baseUrl;
  this.dynapiFunctions = dynapi.functions;
  this.dynapiDocument = dynapi.document;
  this.mapList = new Array();
}

MapManager.prototype.numberMaps = function() {
  return( this.mapList.length );
}

MapManager.prototype.addMap = function( mapFileName, viewPriority, neattoolsIndicator ) {
  var dynImage = this.dynapiFunctions.getImage( this.baseUrl + mapFileName );
  var map = this.mapList[ this.mapList.length ] = new Map( dynImage, viewPriority, neattoolsIndicator );
  return map;
}


MapManager.prototype.getView = function( neattoolsOffset ) {
  for ( var i = 0; i < this.mapList.length; i++ ) {
    m = this.mapList[i];
    if ( m.neattoolsIndicator == neattoolsOffset ) {
      return( m.mapView );
    }
  }
  return 0;
}
